/*
** funcptr.c - short program to demonstrate the
**             use of function pointers within C
*/

#include <stdio.h>

/*
** hugo, resi, sepp: three functions that output
**                   their name when called
*/
int hugo( char *text )
{
    printf( "hugo   : Hello, I'm hugo! (%s)\n", text );
    return( 0 );
}


int resi( char *text )
{
    printf( "resi   : Hello, I'm resi! (%s)\n", text );
    return( 1 );
}


int sepp( char *text )
{
    printf( "sepp   : Hello, I'm sepp! (%s)\n", text );
    printf( "sepp   : but Buhuu!, I'm never called within this program!\n" );
    return( 2 );
}

/*
** call_me: a function that calls one of the the above
**          functions passed as argument
*/
void call_me( int (*me)( char *msg ), char *text )
{
    int result = 0;

    printf( "call_me: calling function at %p with \"%s\"\n",
            me, text );

    /* called the function passed as argument */
    result = (*me)( text );

    printf( "call_me: result=%d\n", result );
}

/*
** the main function
*/
int main( void )
{
    /* this is a variable that can point to one of the
    ** three above functions */
    int (*any_func)( char *msg ) = NULL;

    /* display functions and addresses where in memory they
    ** are located */
    printf( "functions:\n  hugo : %p\n  resi : %p\n  sepp : %p\n",
            hugo, resi, sepp );

    /* assign sepp() to any_func (but DO NOT CALL it */
    any_func = sepp;

    /* call hugo() directly */
    hugo( "direct call" );

    /* assign resi() to any_func, call it */
    any_func = resi;
    (*any_func)( "call via any_func" );

    /* call hugo() via call_me() */
    call_me( hugo, "call via call_me()" );

    /* call resi() via call_me, passed via any_func */
    call_me( any_func, "call via call_me() any_func" );

    return( 0 );
}

