/*
** kackprt.c
**
** short program to demonstrate the use of
** variable arguments within C funtions
*/
#include <stdio.h>
#include <stdarg.h>

static const char *xdigit = "0123456789abcdef";

/*
** putstring - display a string on stdout
*/
void putstring( char *s )
{
    while ( s[0] )
    {

        putchar( s[0] );
        s++;
    }
}

/*
** kackprt()
**
** simle version of printf(), that understands
** only two conversion:
**
** %s ... string conversion
** %h ... hexadecimal conversion
*/
void kackprt( char *format, ... )
{
    va_list ap;

    va_start( ap, format );

    while ( format[0] )
    {
        if ( format[0] == '%' )
        {
            switch ( format[1] )
            {
                int num = 0;

                case 's':
                    /*
                    ** process string argument
                    */
                    putstring( va_arg( ap, char *) ); /* get & print string arg */
                    break;

                case 'x':

                    /*
                    ** process 16-bit hex argument
                    */
                    num = va_arg( ap, int ); /* get numeric argument */
                    putstring( "0x" );
                    putchar( xdigit[ (num & 0xf000) >> 16 ] );
                    putchar( xdigit[ (num & 0x0f00) >>  8 ] );
                    putchar( xdigit[ (num & 0x00f0) >>  4 ] );
                    putchar( xdigit[ (num & 0x000f) >>  0 ] );
                    break;

                default:
                    /*
                    ** process unknown conversion
                    */
                    putchar( format[0] );
                    putchar( format[1] );

            }

            /* process next char */
            format++;
            if ( format[0] )
                format++;
        }
        else
        {
            /*
            ** process normal text
            */
            putchar( format[0] );      /* display current char */
            format++;                  /* process next char */
        }
    }

    va_end( ap );
}

/*
** main function
*/
int main( void )
{
    char *hugo = "hugo";
    char *sepp = "sepp";

    int   resi =   1234; /* dezimal */
    int   hinz =  01234; /* ocal */
    int   kunz = 0x1234;  /* hex */

    /* display some data using kackprt() */
    kackprt( "hugo & resi: \"%s\" & %x\n", hugo, resi );
    kackprt( "kunz & sepp: %x & \"%s\"\n", kunz, sepp );
    kackprt( "hinz & kunz: %x & %x\n", hinz, kunz );

    /* try to display some data using kackprt(), but
    ** with the wrong conversion set:
    ** - the first is a numeric value displayed as a
    **   string, and will interpret the numeric value
    **   as a pointer to char-data.
    ** - the second is a string displayed as a hex-value
    **   and will display the lowbyte of the address
    **   where the string data is located.
    **
    ** NOTE: on systems with memory-protection, this
    **   probably will lead to a memory violation.
    */
    kackprt( "bullshit   : \"%s\" & %x\n", resi, sepp );

    return( 0 );
}
