#include <stddef.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef APTR_TYPEDEF
#define APTR_TYPEDEF
typedef void           *APTR;       /* 32-bit untyped pointer */
#endif
typedef long            LONG;       /* signed 32-bit quantity */
typedef unsigned long   ULONG;      /* unsigned 32-bit quantity */
typedef short           WORD;       /* signed 16-bit quantity */
typedef unsigned short  UWORD;      /* unsigned 16-bit quantity */
#if __STDC__
typedef signed char     BYTE;       /* signed 8-bit quantity */
#else
typedef char            BYTE;       /* signed 8-bit quantity */
#endif
typedef unsigned char   UBYTE;      /* unsigned 8-bit quantity */

#ifdef __cplusplus
typedef char           *STRPTR;     /* string pointer (NULL terminated) */
#else
typedef unsigned char  *STRPTR;     /* string pointer (NULL terminated) */
#endif


/* Types with specific semantics */
typedef void            VOID;
typedef short           BOOL;
typedef unsigned char   TEXT;

#ifndef TRUE
#define TRUE            1
#endif
#ifndef FALSE
#define FALSE           0
#endif
#ifndef NULL
#define NULL            0L
#endif

#define BYTEMASK        0xFF
#define WORDMASK        0xFFFF


typedef const unsigned char *CONSTRPTR;          /* string constants */
typedef unsigned char        STRARR;             /* string array */
typedef unsigned char        CHAR;                /* single character */

/*
** UPTR as an generic pointer. C-math will not operate on UPTR.
** UPTR can be converted to any other pointer and the other way round.
*/
typedef void *UPTR;                    /* generic pointer ( ANSI-def ) */


/*
** compare/delete function type
*/
typedef int  cmp_func( UPTR data1, UPTR data2 );
typedef void del_func( UPTR data );

/*-----------------------*/

#ifndef UGLY_STRING_H
#define UGLY_STRING_H

/*
** ugly/string.h
*/

#define freestr( s ) ugly_freestr(s,__FILE__,__LINE__)
#define ufreestr( s ) freestr(s),s=NULL
#define strclone( s ) ugly_strclone(s,__FILE__,__LINE__)
#define reallocstr( o,n ) ugly_reallocstr(o,n,__FILE__,__LINE__)

/* options for strenum */
#define STEN_CASE   0 /* case sensitive search */
#define STEN_NOCASE 1 /* case insensitive search */

/*
** ugly/string.c
*/
char ch2str_buffer[2];                 /* internal buffer for _ch2str() */
char num2str_buffer[10];               /* internal buffer for _long2str() */



/*
** strclone
**
** create a clone of a string 
**     (copy of string using its own memory area)
**
** params: *oldstr...ptr to string to clone
** result: ptr to copy of string;
** errors: out of memory: returns NULL
**
*/
STRPTR ugly_strclone( CONSTRPTR oldstr, STRPTR file, ULONG line )
{
    STRPTR newstr = NULL;

    if ( oldstr ) {

        newstr =                           /* alloc mem for clone */
#ifdef UMEM_TRACKING
            ugly_malloc_tracking( strlen( oldstr ) +1, file, line );
#else
            umalloc(  strlen( oldstr ) +1 );
#endif

        if ( newstr )                      /* alloc sucessful? */
            strcpy( newstr, oldstr );      /* Y-> copy data */

    }

    return( newstr );                  /* return result */
}


/*
** upstr: convert a string to upper case
**
** params: *s...string to convert
**
*/
STRPTR upstr( STRPTR s )
{
    STRPTR s_old = s;

    if (s)
        for (; *s != '\0'; s++)
            *s = toupper(*s);

    return s_old;
}

/*
** upstrcmp
**
** string compare, ignore case
**
** params: *s1, *s2...ptr to strings to compare
** result: -1: s1 < s2
**          0: s1 = s2
**          1: s1 > s2
**
*/
int upstrcmp( CONSTRPTR s1, CONSTRPTR s2 )
{
    int equal;                         /* result of _strcp() */
    unsigned char c1, c2;              /* chars currently comparing */
    size_t i = 0;                      /* string index counter */

    do {
        c1 = toupper( s1[i] );
        c2 = toupper( s2[i] );
        i++;
    } while ( c1 && c2 && ( c1 == c2 ) );

    if ( c1 < c2 )
        equal = -1;                    /* s1 < s2 */
    else if ( c1 > c2 )
        equal = +1;                    /* s1 > s2 */
    else
        equal =  0;                    /* s1 = s2 */

    return( equal );                   /* return result */
}


/*
** upstrncmp
**
** string compare, ignore case, max. n chars
**
** params: *s1, *s2...ptr to strings to compare
**         n..........max. nr. of chars to compare
** result: -1: s1 < s2
**          0: s1 = s2
**          1: s1 > s2
**
*/
int upstrncmp( CONSTRPTR s1, CONSTRPTR s2, size_t n )
{
    int equal;                         /* result of _strcp() */
    unsigned char c1, c2;              /* char of string currently comparing */
    size_t i = 0;                      /* string index counter */

    do {
        c1 = toupper( s1[i] );
        c2 = toupper( s2[i] );
        i++;
    } while ( c1 && c2 && ( c1 == c2 ) && ( i < n) );

    if ( c1 < c2 )
        equal = -1;                    /* s1 < s2 */
    else if ( c1 > c2 )
        equal = +1;                    /* s1 > s2 */
    else
        equal =  0;                    /* s1 = s2 */

    return(equal);                     /* return result */
}




/*
** freestr
**
** free memory used by a string
**
** params: s..string to free
**
** NOTE: strings created with strclone() or reallocstr
**       should be freed with this function instead of
**       ufree(). this will avoid problems when using
**       the memory-tracking feature of ugly.o
*/
void ugly_freestr( STRPTR s, STRPTR file, ULONG line )
{
#ifdef UMEM_TRACKING
    ugly_free( s, file, line );
#else
    ufree( s );
#endif
}

/*
** reallocstr
**
** free memory used by string, clone new data
**
** params: **oldstr...prt to ptr of old string
**          *newstr...ptr to new string data
** errors: out of mem: returns NULL (old data are lost)
**
** IMPORTANT: old string has to be initialies like "oldstr = NULL;" before !!
**            first call like "reallocstr( &oldstr, "<new data>" );"
*/
void ugly_reallocstr( STRPTR *oldstr,  CONSTRPTR newstr, STRPTR file, ULONG line )
{
#ifdef UMEM_TRACKING
    ugly_freestr( *oldstr, file, line );                   /* free old string */
    *oldstr = ugly_strclone( newstr, file, line );         /* clone new string */
#else
    ufree( *oldstr );                            /* free old string */
    *oldstr = strclone( newstr );                /* clone new string */
#endif
}
/*
** ch2str
**
** convert single char to zero terminated string
**
** params: ch...char to convert
** result: ptr to internal buffer, where a string containning
**         "<ch>\0" is located. 
**
** USAGE : buffer is rewritten at next call of _chstr(), so
**         be sure to clone string if you need it for a longer
**         time.
**         _ch2str() is mostly designed for parameter conversion
**         on string funtions, e.g s = strclone( ch2str( 'x' ) );
**
*/
STRPTR ch2str( const char ch )
{
    ch2str_buffer[0] = ch;
    ch2str_buffer[1] = '\0';

    return ch2str_buffer;
}

/*
** ustrrpbrk
**
** searches for last occurence of a char of _set in _str
**
** params: str...string to examine
**         set...chars to search for
** result: ptr where char of _set occured in _str
**         or NULL if none occurence found
**
** NOTE:   this is a clone of none-ansi-c function _strrpbrk()
**
*/
STRPTR ustrrpbrk( CONSTRPTR str, CONSTRPTR set )
{
    size_t i;
    STRPTR result = NULL;

    if ( str ) {

        i = strlen( str )-1;

        if ( i >= 0 ) {
        
            while ( ( i ) && ( strchr( set, str[i] ) == NULL ) )
                i--;

            if ( strchr( set, str[i] ) )
                result = ( STRPTR ) &(str[i]);
        }

    }

    return result;

}


/*
** str2long
**
** convert string expression to long value
**
** params: s....string to convert
**         num..ptr to destination long var
** result: TRUE, if sucessful
** errors: returen FALSE
**
*/
BOOL str2long( STRPTR s, LONG *num )
{
    BOOL conv_ok = FALSE;

    if ( sscanf( s, "%d", num ) ) {

        conv_ok = TRUE;

    }

    return conv_ok;
}


/*
** long2str
**
** convert long value to string expresion
**
** params: num...value to convert
** result: ptr to converted string
** errors: returns NULL
**
*/
STRPTR long2str( LONG num )
{
    STRPTR result_str = NULL;

    if ( sprintf( num2str_buffer, "%d", num ) ) {

        result_str = num2str_buffer;

    }

    return result_str;
}

/*
** strenum
**
** find a substring in an enumerator string
**
** params: str...substring to search for
**         set...set of legal substrings
**         sep...substring separator (eg "|")
** result:  0..not found
**         >0..index of substring found
**         <0..out of mem
** errors: out of mem: returns -1
**
** example: enumstr( "sepp", "hugo|sepp|resi", '|', STEN_CASE ) -> 2
**
** NOTE: calls strtok()
*/
LONG strenum( STRPTR str, STRPTR set, char sep, BYTE options )
{
    STRPTR s     = strclone( set );
    LONG   found = 0;

    if ( s ) {

        STRPTR nxtstr = strtok( s, ch2str( sep ) );
        LONG   count  = 1;

        while ( !found && nxtstr ) {

            if ( options & STEN_NOCASE ) {
                if ( !upstrcmp( str, nxtstr ) )
                    found = count;
            } else if ( !strcmp( str, nxtstr ) )
                found = count;

            count++;
            nxtstr = strtok( NULL, ch2str( sep ) );

        }

        ufreestr( s );

    } else
        found = -1;

    return( found );
}

/*
** ugly/memory.h
*/

typedef struct uglymem {

    struct uglymem *next;
    void  *ptr;      /* ptr to mem area allocated before */
    size_t size;     /* size of this area */
    STRPTR file;     /* filename from which call came */
    ULONG  line;     /* line num in this file */
    UBYTE  fillchar; /* fill character for wall */

} UGLYMEM;

#ifdef UMEM_TRACKING

/* ugly function calls with memory tracking ENABLED */
#define umalloc(size) ugly_malloc_tracking( size, __FILE__, __LINE__ )
#define ufree(ptr)    if ( ptr ) { ugly_free( ptr, __FILE__, __LINE__ ); ptr = NULL; }
#define urealloc(ptr,size) ugly_realloc( ptr, size, __FILE, __LINE__ );
#define ucalloc(count,size) ugly_calloc( count,size,__FILE__,__LINE );

#define umem_report(msg) uglymem_report( msg, __FILE__, __LINE__, __DATE__, __TIME__ )
#define umem_stats(msg) uglymem_stats( msg, __FILE__, __LINE__, __DATE__, __TIME__ )

#define atexit_uglymemory atexit_uglymemory_real

#else

/* ugly function calls with memory tracking disabled */
#define umalloc(size) ugly_malloc_notracking(size)
#define ufree(ptr)    if ( ptr ) { free(ptr); ptr=NULL; } /* TODO: use only free() */
#define urealloc(ptr,size)  realloc( ptr, size );
#define ucalloc(count,size) calloc( count,size )

#define umem_report(msg)    /* nufin */
#define umem_stats(msg)     /* nufin */

#define atexit_uglymemory atexit_uglymemory_dummy

#endif


/*
** size of wall build around every memory block
*/
#define UMEM_WALLSIZE 8

UGLYMEM *first = NULL;

UBYTE deadbeef[4] = { 0xDE, 0xAD, 0xBE, 0xEF }; /* used to fill mem after free() */
UBYTE deadfood[4] = { 0xDE, 0xAD, 0xF0, 0x0D }; /* used to fill mem after malloc() */

UBYTE ugly_fillchar = 0x81;


/*
**
**
**
**
**
**
**
**
**
**
** CONTINUE
**
**
**
**
**
**
**
**
**
**
*/
ULONG ugly_umalloc_count      = 0; /* num. of calls to umalloc()/ucalloc() */
ULONG ugly_ufree_count        = 0; /* num. of calls to ufree() */
ULONG ugly_umalloc_count_fail = 0; /* num. of failed calls to umalloc() */
ULONG ugly_ufree_count_fail   = 0; /* num. of failed calls to ufree() */
ULONG ugly_maxmem_usage       = 0; /* maximum memmory used */
ULONG ugly_curmem_usage       = 0; /* current memory used */
ULONG ugly_maxnod_usage       = 0; /* maximum num. of memmory nodes used */
ULONG ugly_curnod_usage       = 0; /* current num. of memory nodes used */

/* forwar reference */
void *ugly_malloc_notracking( size_t size );

/* function pointer for nomem-handler */
BOOL (*ugly_nomem_handler)( size_t size ) = NULL;

/*
** find_umem( UGLYMEM *umem )
*/
UGLYMEM *find_umem( void *mem )
{
    UGLYMEM *nxtum = first;
    UGLYMEM *found = NULL;

    while ( nxtum && (!found) ) {

        if ( nxtum->ptr == mem )
            found = nxtum;
        nxtum = nxtum->next;

    }

    return ( found );
}

/*
** find_prev
*/
UGLYMEM *find_prev( UGLYMEM *umem )
{
    UGLYMEM *prev  = first;
    UGLYMEM *pprev = NULL;
    BOOL     found = FALSE;

    while ( prev && (!found) ) {
        found = ( prev == umem );
        if ( !found ) {
            pprev = prev;
            prev  = prev->next;
        }

    }

    return ( pprev );
}


/*
** del_uglymem
**
** free an uglymem-entry
*/
void del_uglymem( UGLYMEM *umem )
{
    UGLYMEM *prev = find_prev( umem );


    /* unlink from list */
    if ( prev ) {
        prev->next = umem->next;
    } else {
        first = umem->next;
    }

    /* free data */
    free( umem->ptr );
    umem->size = 0;
    umem->file = NULL;
    umem->line = 0;

    free( umem );
}

/*
** fill_mem4
*/
void fill_mem4( void *mem, size_t size, UBYTE value[4] )
{
    size_t i;

    for ( i=0; i<size; i++ )
        ( ((UBYTE *) mem)[i] ) = value[ i%4 ];
}

/*
** new uglymem
**
** alloc & init a new entry of ugly mem
*/
UGLYMEM *new_uglymem( size_t memsize, STRPTR memfile, ULONG memline )
{
    UGLYMEM *newmem = (UGLYMEM *) malloc( sizeof(UGLYMEM) );

    if ( newmem ) {

        newmem->ptr = ugly_malloc_notracking( memsize );
        if ( newmem->ptr ) {


            /* link to list */
            newmem->next = first;
            first = newmem;

            /* init data */
            newmem->size = memsize;
            newmem->file = memfile;
            newmem->line = memline;
            newmem->fillchar = ugly_fillchar;

            /* update fillchar */
            if ( ugly_fillchar == 0xff )
                ugly_fillchar = 0x81;
            else
                ugly_fillchar++;

            /* fill new mem area with $DEADF00D */
            fill_mem4( newmem->ptr, memsize, deadfood );

        } else
            free( newmem );
    }

    return ( newmem );
}


void uglymem_message( STRPTR msg )
{
    fprintf( stderr, "%s\n", msg );
}

void ugly_memdump( void *ptr, size_t size )
{
    STRPTR data = (STRPTR) ptr;

    /* limit size */
    if ( size > 16 )
        size = 16;

    fprintf( stderr, "  %08p:", ptr );
    if ( data ) {

        size_t i;

        /* hex dump */
        for ( i=0; i<size; i++ ) {

            if ( !(i%4) )
                fprintf( stderr, " " );
            fprintf( stderr, "%02x", data[i] );

        }

        fprintf( stderr, "  \"" );
        /* ascii dump */
        for ( i=0; i<size; i++ )
            if ( data[i]<' ' )
                fprintf( stderr, "." );
            else
                fprintf( stderr, "%c", data[i] );
        fprintf( stderr, "\"\n" );

    } else
        fprintf( stderr, "NULL\n" );

}

void uglymem_meminfo( void *ptr, STRPTR file, ULONG line )
{
    fprintf( stderr, "  in \"%s\" (%d) at %08p\n", file, line, ptr );
}

void umem_info( UGLYMEM *umem )
{
    fprintf( stderr, "  %06d (0x%04x) bytes from \"%s\" (%d) at %08p\n",
             umem->size, umem->size, umem->file, umem->line, umem->ptr );
}

/*
**-------------------------------------
** memory statistics functions
**-------------------------------------
*/

/*
** ugly_mem_report
**
** displaly all memory nodes currently allocated
*/
void uglymem_report( STRPTR msg, STRPTR file, ULONG line, STRPTR date, STRPTR time  )
{
    UGLYMEM *umem = first;

    if ( umem ) {

        /* report header */
        fprintf( stderr, "Memory Report (%s)\n", msg );
        if ( file )
            fprintf( stderr, "(\"%s\" (%d), at %s, %s)\n",
                             file, line, date, time );

        /* print all elements */
        while ( umem ) {

            umem_info( umem );
            ugly_memdump( umem->ptr, umem->size );
            umem = umem->next;
        }
    }
}

/*
** ugly_mem_stats
**
** display memory statistics (nodes & size allocated)
*/
void uglymem_stats( STRPTR msg, STRPTR file, ULONG line, STRPTR date, STRPTR time  )
{
    /* statistics header */
    fprintf( stderr, "Memory statistics (%s)\n", msg );
    if ( file )
        fprintf( stderr, "(\"%s\" (%d), at %s, %s)\n",
                         file, line, date, time );

    /* memory statistics */
    fprintf( stderr, "  bytes used: %d (max: %d)\n",
                     ugly_curmem_usage, ugly_maxmem_usage );
    fprintf( stderr, "  nodes used: %d (max: %d)\n",
                     ugly_curnod_usage, ugly_maxnod_usage );
    fprintf( stderr, "  calls to: umalloc(%d)   ufree(%d)\n",
                     ugly_umalloc_count, ugly_ufree_count );

}


/*
**-------------------------------------
** atexit functions
**-------------------------------------
*/


/*
** atexit_uglymemory_real
*/
void atexit_uglymemory_real( void )
{
    ULONG mem_lost = ugly_curmem_usage;
    uglymem_report( "at exit:  MEMORY LEAK detected!",
                    NULL, 0, NULL, NULL );
    uglymem_stats( "[exit]", NULL, 0, NULL, NULL );

    /* release all lost mem */
    while ( first )
        del_uglymem( first );

    if ( mem_lost )
        fprintf( stderr, "\n%d bytes of memory lost!\n",  mem_lost );
}


/*
** atexit_uglymemory_dummy
*/
void atexit_uglymemory_dummy( void )
{
    /* do nufin */
}

/*
**-------------------------------------
** memory handling functions
**-------------------------------------
*/

/*
** ugly_malloc_notracking
*/
void *ugly_malloc_notracking( size_t size )
{
    void *mem;
    BOOL  retry;

    do {

        mem = malloc( size );
        if ( !mem && ugly_nomem_handler ) {

            /* call nomem-handler */
            retry = (*ugly_nomem_handler)( size );
            if ( !retry )
                exit( EXIT_FAILURE ); /* abort programm */

        } else
            retry = FALSE;

    } while ( retry );

    return( mem );
}


/*
** ugly_malloc_tracking
*/
void *ugly_malloc_tracking( size_t size, STRPTR file, ULONG line )
{
    void    *mem = NULL;
    UGLYMEM *umem = NULL;

    if ( size ) {

        /* update num. of calls to umalloc() */
        ugly_umalloc_count++;

        /* alloc new uglymem */
        umem = new_uglymem( size, file, line );
        if ( umem ) {

            mem = umem->ptr;

            /* update memory usage and num of nodes */
            ugly_curmem_usage += size;
            if ( ugly_curmem_usage > ugly_maxmem_usage )
                ugly_maxmem_usage = ugly_curmem_usage;
            ugly_curnod_usage++;
            if ( ugly_curnod_usage > ugly_maxnod_usage )
                ugly_maxnod_usage = ugly_curnod_usage;

        }
    } else {

        /* zero-alloc */

        /* update num. of failed calls to umalloc() */
        ugly_umalloc_count_fail++;

        uglymem_message( "MALLOC: zero-sized allocation" );
        uglymem_meminfo( NULL, file, line );

    }

    return (mem);
}

/*
** ugly_free
*/
void ugly_free( void *ptr, STRPTR file, ULONG line )
{
    if ( ptr ) {

        UGLYMEM *umem = find_umem( ptr );

        if ( umem ) {

            /* update num. of calls to ufree() */
            ugly_ufree_count++;

            /* fill memory with $DEADBEEF */
            fill_mem4( umem->ptr, umem->size, deadbeef );

            /* update memory usage */
            ugly_curmem_usage -= umem->size;

            /* remove node from mem-list */
            del_uglymem( umem );
            ugly_curnod_usage--;

        } else {

            /* ptr has never been allocated */

            /* update num. of calls to ufree() */
            ugly_ufree_count_fail++;

            /* -> error message */
            uglymem_message( "FREE: memory never allocated" );
            uglymem_meminfo( ptr, file, line );
        }
    }
}

/*
** ugly_realloc
*/
void *ugly_realloc( void *ptr, size_t size, STRPTR file, ULONG line )
{
    ugly_free( ptr, file, line );                          /* free old mem */
    return( ugly_malloc_tracking( size, file, line ) );    /* alloc new mem */
}

/*
** ugly_calloc
*/
void *ugly_calloc( size_t count, size_t size, STRPTR file, ULONG line )
{
    /* alloc new mem */
    void *mem = ugly_malloc_tracking( count*size, file, line );

    /* fill mem with zero */
    if ( mem )
        memset( mem, 0, size*count );

    return( mem );
}

